/*
  DcMotor.cpp - Libreria per gestire i movimenti di un motore DC con il modulo L298N
  Autore:   Andrea Lombardo
  Sito web: http://www.lombardoandrea.com
*/

#include "DcMotor.h"

typedef void (*CallBackFunction) ();

DcMotor::DcMotor(){

}

void DcMotor::begin(uint8_t pinEnable, uint8_t pinIN1, uint8_t pinIN2){
  _pinEnable = pinEnable;
  _pinIN1 = pinIN1;
  _pinIN2 = pinIN2;
  _pwmVal = 100;   //velocidad
  _isMoving = false;
  _canMove = true;
  _lastMs = 0;

  pinMode(_pinEnable, OUTPUT);
  pinMode(_pinIN1, OUTPUT);
  pinMode(_pinIN2, OUTPUT);
}

void DcMotor::move(uint8_t direction, unsigned short pwmVal){     // direccion y velocidad
    _pwmVal = pwmVal; // seteo de la velociad
    run(direction);                 
}

void DcMotor::setSpeed(unsigned short pwmVal){
  _pwmVal = pwmVal;
}

unsigned short DcMotor::getSpeed(){
  return _pwmVal;
}

void DcMotor::forward(){
  digitalWrite(_pinIN1, HIGH);
  digitalWrite(_pinIN2, LOW);

  analogWrite(_pinEnable, _pwmVal);

  _isMoving = true;
}


void DcMotor::forwardFor(unsigned long delay, CallBackFunction callback){
  if ((_lastMs == 0) && _canMove) {
    _lastMs = millis();
    this->forward();
  }

  if (((millis() - _lastMs) > delay) && _canMove) {
    this->stop();
    _lastMs = 0;
    _canMove = false;

    callback();

  }
}

void DcMotor::forwardFor(unsigned long delay){
  this->forwardFor(delay, fakeCallback);
}


void DcMotor::backward(){
  digitalWrite(_pinIN1, LOW);
  digitalWrite(_pinIN2, HIGH);

  analogWrite(_pinEnable, _pwmVal);

  _isMoving = true;
}

void DcMotor::backwardFor(unsigned long delay, CallBackFunction callback){

  if ((_lastMs == 0) && _canMove) {
    _lastMs = millis();
    this->backward();
  }

  if (((millis() - _lastMs) > delay) && _canMove) {
    this->stop();
    _lastMs = 0;
    _canMove = false;

    callback();

  }
}

void DcMotor::backwardFor(unsigned long delay){
  this->backwardFor(delay, fakeCallback);
}

void DcMotor::run(uint8_t direction){
  switch(direction){
    case BACKWARD:
      this->backward();
    break;
    case FORWARD:
      this->forward();
    break;
  }
}

void DcMotor::stop(){
  digitalWrite(_pinIN1, LOW);
  digitalWrite(_pinIN2, LOW);
  analogWrite(_pinEnable, 255);
  _isMoving = false;
}


void DcMotor::reset(){
  _canMove = true;
}

boolean DcMotor::isMoving(){
  return _isMoving;
}

void DcMotor::fakeCallback(){
}
