/*
 * Stepper.h - Stepper library for L298N
 *
 * Original library        (0.1)   by Tom Igoe.
 * Two-wire modifications  (0.2)   by Sebastian Gassner
 * Combination version     (0.3)   by Tom Igoe and David Mellis
 * Bug fix for four-wire   (0.4)   by Tom Igoe, bug fix from Noah Shibley
 * High-speed stepping mod         by Eugene Kozlenko
 * Timer rollover fix              by Eugene Kozlenko
 * Five phase five wire    (1.1.0) by Ryan Orendorff
 * Six secuence correction        by Andres Lopez
 * The sequence of control signals for 4 control wires is as follows:
 *
 * Step   C0 C1 C2 C3
 *    1 - 1  1  0  0
 *    2 - 0  1  1  0
 *    3 - 0  0  1  1
 *    4 - 1  0  0  1
 */

// ensure this library description is only included once
#ifndef _Stepper_h
#define _Stepper_h

// library interface description
class Stepper {
  public:
    // constructors:
    Stepper();
    void begin(int number_of_steps, int motor_pin_1, int motor_pin_2,
                                     int motor_pin_3, int motor_pin_4);

    // speed setter method:
    void setSpeed(int whatSpeed);

    // mover method:
    void step(int number_of_steps);

  private:
    void stepMotor(int this_step);
    int direction;             // Direction of rotation
    unsigned long step_delay;  // delay between steps, in ms, based on speed
    int number_of_steps;      // total number of steps this motor can tak
    int step_number = 0;          // which step the motor is on

    // motor pin numbers:
    int motor_pin_1;
    int motor_pin_2;
    int motor_pin_3;
    int motor_pin_4;

    unsigned long last_step_time; // time stamp in us of when the last step was taken
};

#endif // end _Stepper_h
